<?php
error_reporting(E_ALL & ~E_NOTICE);
ini_set('display_errors', true);

// version 06/24/03
// written by Lars Torben Wilson [torben@php.net]
// Daevid Vincent (daevid@daevid.com) fixed a bug where the ; was being stored with the array elements
// Daevid Vincent (daevid@daevid.com) fixed a bug where comments weren't being stripped out of the dhcpd.leases file
//				      so it was possible that the first lease in the file wouldn't be 'valid' and was omitted.
// Daevid Vincent (daevid@daevid.com) fixed a bug where the start and end could have a number other than 1 (like 3),
//                    so now the lease is in the key 'starts' or 'ends' vs. 'starts 1' or 'ends 3'

class LeaseParser {
    // List designators valid within lease blocks here.
    // Add/modify/remove to suit.
    var $_valid_items = 
    array('starts \d',
          'ends \d',
          'binding state active',
          'next binding state',
          'hardware ethernet',
          'uid',
          'client-hostname');

    // Stores any errors we generate during the run. Theses can be output
    // directly by calling showErrors();
    var $_errors;

    // Storage for the contents of the lease file while it's being working on.
    var $_file_contents;

    // Stores the parsed lease array.
    var $_leases;

    // Put any options you want to define in here, and use setOption() to modify it.
    var $_options = array('file_name' => '');


    // Private method. Attempts to load the leases file.
    function _loadFile() {
        if (!$fp = @fopen($this->_options['file_name'], 'r')) {
            $this->_errors[] = "LeaseParser::_loadFile(): Failed to load {$this->_options['file_name']}";
            return false;
        }

        #if (!$file = @fread($fp, filesize($this->_options['file_name']))) {
        #    $this->_errors[] = "LeaseParser::_loadFile(): Failed to read {$this->_options['file_name']}";
        #    return false;
        #}
       	#$this->_file_contents =& $file;

        while( $line = fgets($fp, 1024) )
        	if ($line[0] != "#") $this->_file_contents .= $line;  // strip out # comments

        fclose ($fp);

        return true;
    }

    
    // Private method. Attempts to parse the loaded file contents into a usable format.
    function _parseFile() {
        if (empty($this->_file_contents)) {
            $this->_errors[] = "LeaseParser::_parseFile(): No leases file data to parse.";
            return false;
        }

        $f_temp = explode('}', $this->_file_contents);
	#print_r($f_temp);

        $leases = array();

        foreach ($f_temp as $lease_record) {
//             echo "<hr>";
//             print_r($lease_record);
            if (!preg_match('/^\s*lease (([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3}))/i', $lease_record, $lease_matches)) {
                continue;
            }
//             echo "{$lease_matches[1]}\n";
            $leases[$lease_matches[1]] = array();
            $working_lease =& $leases[$lease_matches[1]];
            foreach ($this->_valid_items as $valid_item) {
                $matches = array();
                $pattern = "/ *({$valid_item}) (.*);\n/i";
//                 echo "Checking for $valid_item; $pattern\n";
                if (!preg_match_all($pattern, $lease_record, $matches, PREG_SET_ORDER)) {
                    continue;
                }
//                 echo "-----\n";
//                 print_r($matches);
				 
				 //remove the number after the lease start/end key word (also, notice the !== operator)
				 if ( strpos($matches[0][1],"starts") !== false)
				 	$matches[0][1] = "starts";
				 else if ( strpos($matches[0][1],"ends") !== false)
				 	$matches[0][1] = "ends";
				 
                $working_lease[$matches[0][1]] = $matches[0][2];
            }
        }

//         print_r($leases);

        $this->_leases = $leases;

        return true;
    }


    // If a leases file has been loaded and parsed, this will return the generated array.
    function getLeases() {
        if (!empty($this->_leases)) {
            return $this->_leases;
        }
    }


    // Called by the user to load and parse a lease file. The filename must have already been set.
    function parse() {
        if (!$this->_loadFile()) {
            $this->_errors[] = "LeaseParser::parse(): Could not load leases file.";
            return false;
        }
        if (!$this->_parseFile()) {
            $this->_errors[] = "LeaseParser::parse(): Could not parse leases file.";
            return false;
        }
        return true;
    }


    // Sets an option. The option must be defined in $_options.
    function setOption($option, $value) {
        if (!isset($this->_options[$option])) {
            $this->_errors[] = "LeaseParser::setOption(): Attempt made to set illegal option.";
            return false;
        }
        $old_val = $this->_options[$option];
        $this->_options[$option] = $value;
        return $old_val;
    }


    // Dump any errors to output.
    function showErrors() {
        if (empty($this->_errors)) {
            return false;
        }
        print_r($this->_errors);
        return true;
    }
    

    // Null contstructor.
    function LeaseParser() {  }
}

$lp =& new LeaseParser();
$lp->setOption('file_name', '/var/lib/dhcp/dhcpd.leases');
$lp->parse();
$leases = $lp->getLeases();
//print_r($leases);
$lp->showErrors();

//echo "\n".$leases['192.168.1.6']['client-hostname']."\n";
?>

